VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "Statistics"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
Attribute VB_Description = "Simple statistical analysis class.  It doesn't do a lot of fancy things, but it will return primary single-variable statistical measures for a sample and also exposes factorial, array sum, array product, and sorting routines."
Attribute VB_Ext_KEY = "SavedWithClassBuilder6" ,"Yes"
Attribute VB_Ext_KEY = "Top_Level" ,"Yes"

' Copyright (C) 2003  Alex K. Angelopoulos, aka@mvps.org
'
' This library is free software; you can redistribute it and/or
' modify it under the terms of the GNU Lesser General Public
' License as published by the Free Software Foundation; either
' version 2.1 of the License, or (at your option) any later version.
'
' This library is distributed in the hope that it will be useful,
' but WITHOUT ANY WARRANTY; without even the implied warranty of
' MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
' Lesser General Public License for more details.
'
' You should have received a copy of the GNU Lesser General Public
' License along with this library; if not, write to the
' Free Software Foundation, Inc.
' 59 Temple Place, Suite 330
' Boston, MA  02111-1307  USA
'

Option Explicit

' Global values for statistical measures
' these are derived when the
Dim m_Data() As Variant ' array of sample set data points
Dim m_Sum As Double ' sum of all data
Dim m_Mean As Double
Dim m_Median As Double ' median value of data set
Dim m_Sigma As Double ' standard deviation of data
Dim m_Variance As Double ' sample variance
Dim m_absdev As Double ' absolute deviation
Dim m_Kurtosis As Double '
Dim m_Skewness As Double '
Dim m_Range As Double '
Dim m_Minimum As Double '
Dim m_Maximum As Double '
Dim m_Count As Integer '
Dim n_1 As Long ' This is the "n-1" count for use in stats

Public Function factorial(ByVal objectCount As Long, _
      Optional ByVal samplesize As Long = 1) As Long
Attribute factorial.VB_Description = "returns the factorial of objectCount; samplesize covers how many are taken at a time for the permutation calculation."

  Dim i As Long
  For i = samplesize To objectCount
    factorial = factorial * i
  Next
End Function

Public Function ArraySum(ByVal a As Variant) As Double
Attribute ArraySum.VB_Description = "Calculates sum of the variant array submitted to it."
  Dim i As Long
  For i = LBound(a) To UBound(a)
    ArraySum = ArraySum + a(i)
  Next
End Function

Public Function ArrayProduct(ByVal a As Variant) As Double
Attribute ArrayProduct.VB_Description = "Calculates product of the variant array submitted to it."
  Dim i As Long
  ArrayProduct = 1
  For i = LBound(a) To UBound(a)
    ArrayProduct = ArrayProduct * a(i)
  Next
End Function

Public Property Let NewDatum(ByVal value As Double)
Attribute NewDatum.VB_Description = "Allows addition of a single datum."
  n_1 = n_1 + 1
  ReDim Preserve m_Data(n_1)
  m_Data(n_1) = value
End Property

Public Property Let dataSet(ByVal a As Variant)
Attribute dataSet.VB_Description = "Allows you to add an array of sample data at once."
  n_1 = UBound(a)
  ReDim m_Data(n_1)
  m_Data = a
End Property

Private Sub Class_Initialize()
  n_1 = -1
End Sub

Public Sub Analyze()
Attribute Analyze.VB_Description = "Initiates analysis of a complete data sample. Must be called before any of the statistical properties can be accessed."
  Dim j As Long
  Dim MedianHigherIndex As Long
  Dim MedianLowerIndex As Long
  Dim MedianIndex As Double

  ' sort the data; then get the sum, sample count, and average
  ' for rest of statistical measures
  ShellSortArray m_Data
  m_Sum = ArraySum(m_Data)
  m_Minimum = m_Data(LBound(m_Data))
  m_Maximum = m_Data(UBound(m_Data))
  m_Range = m_Data(UBound(m_Data)) - m_Data(LBound(m_Data))

  ' The standard n-1 sampling can be done easily by subtracting
  ' the lower bound from the upper; 0-based arrays are "natural"
  ' counters for stats.
  m_Count = n_1 + 1
  m_Mean = m_Sum / m_Count

  MedianIndex = (UBound(m_Data) - LBound(m_Data)) / 2 + LBound(m_Data)
  If MedianIndex \ 1 = MedianIndex Then
    m_Median = m_Data(MedianIndex)
  Else
    MedianLowerIndex = MedianIndex \ 1
    MedianHigherIndex = MedianLowerIndex + 1
    m_Median = (m_Data(MedianLowerIndex) _
        + m_Data(MedianLowerIndex)) / 2
  End If

  ' Calculate the variance of the sample
  ' \Hat\sigma^2 = (1/(N-1)) \sum (x_i - \Hat\mu)^2
  Dim tmpSum As Double: tmpSum = 0
  For j& = LBound(m_Data) To UBound(m_Data)
    tmpSum = tmpSum + ((m_Data(j&) - m_Mean) ^ 2)
  Next
  m_Variance = tmpSum / n_1
  ' Calculate standard deviation
  m_Sigma = Sqr(m_Variance)

  ' Calculate the variance of the sample
  ' \Hat\sigma^2 = (1/(N-1)) \sum (x_i - \Hat\mu)^2
  For j& = LBound(m_Data) To UBound(m_Data)
    tmpSum = tmpSum + Abs((m_Data(j&) - m_Mean))
  Next
  m_absdev = tmpSum / n_1




  ' Determine skewness and kurtosis
  ' note that this depends on the standard deviation m_Sigma
  tmpSum = 0#
  For j& = LBound(m_Data) To UBound(m_Data)
    tmpSum = tmpSum + (((m_Data(j&) - m_Mean) / m_Sigma) ^ 3)
  Next
  m_Skewness = (tmpSum / n_1)

  tmpSum = 0#
  For j& = LBound(m_Data) To UBound(m_Data)
    tmpSum = tmpSum + (((m_Data(j&) - m_Mean) / m_Sigma) ^ 4)
  Next
  m_Kurtosis = (tmpSum / n_1) - 3

  ' m_Median =
  'm_Mode =
  'm_Sample_Variance =
End Sub

Function ShellSortArray(aData As Variant) As Variant
Attribute ShellSortArray.VB_Description = "Does a quicksort of the data submitted to it."
  '***********************
  ' Purpose: Sorts an array using the QuickSort method
  '
  ' Inputs: aData() : the array to be sorted.
  ' strTestRelationship : a string representation of the boolean relationship of
  ' two arbitrary array elements, X1 and X2. The relationship
  ' is true if the elements are in the correct order.
  '
  ' Source: Adapted from a VB routine ShellSort from the CodeBank at
  '         http://www.vb2themax.com written by the vb2themax team
  '         (D.Esposito, F.Balena, et.al.)
  '
  ' This routine requires VBScript version 5.
  '***********************
  Dim value As Double
  Dim index As Long, index2 As Long
  Dim firstEl As Long
  Dim lastEl As Long
  Dim distance As Long
  Dim numEls As Long

  ' account for optional arguments
  lastEl = UBound(aData)
  firstEl = LBound(aData)

  numEls = lastEl - firstEl + 1
  ' find the best value for distance
  Do
    distance = distance * 3 + 1
  Loop Until distance > numEls

  Do
    distance = distance \ 3
    For index = distance + firstEl To lastEl
      value = aData(index)
      index2 = index
      Do While value < aData(index2 - distance)
        aData(index2) = aData(index2 - distance)
        index2 = index2 - distance
        If index2 - distance < firstEl Then Exit Do
      Loop
      aData(index2) = value
    Next
  Loop Until distance = 1
  ShellSortArray = aData
End Function

Public Property Get Mean() As Double
Attribute Mean.VB_Description = "Statistical mean of the data sample. Not available until the Analyze method has been called."
  Mean = m_Mean
End Property

Public Property Get Median() As Double
Attribute Median.VB_Description = " Not available until the Analyze method has been called."
  Median = m_Median
End Property

Public Property Get Sigma() As Double
Attribute Sigma.VB_Description = "Returns summation of all data. Not available until the Analyze method has been called."
  Sigma = m_Sigma
End Property

Public Property Get Kurtosis() As Double
Attribute Kurtosis.VB_Description = " Not available until the Analyze method has been called."
  Kurtosis = m_Kurtosis
End Property

Public Property Get Skewness() As Double
Attribute Skewness.VB_Description = " Not available until the Analyze method has been called."
  Skewness = m_Skewness
End Property

Public Property Get Range() As Double
Attribute Range.VB_Description = "Returns the range from minimum to maximum in the data set. Not available until the Analyze method has been called."
  Range = m_Range
End Property

Public Property Get Minimum() As Double
Attribute Minimum.VB_Description = "Returns the minimum value from the data set. Not available until the Analyze method has been called."
  Minimum = m_Minimum
End Property

Public Property Get Maximum() As Double
Attribute Maximum.VB_Description = "Returns the maximum value from the data set. Not available until the Analyze method has been called."
  Maximum = m_Maximum
End Property

Public Property Get Count() As Long
Attribute Count.VB_Description = "Returns the size of the data set. Not available until the Analyze method has been called."
  Count = m_Count
End Property


Public Property Get AbsDev() As Double
Attribute AbsDev.VB_Description = "Returns the absolute deviation of the data set. Not available until the Analyze method has been called."
  AbsDev = m_absdev
End Property



Public Property Get ReferenceDataSetUrl()
Attribute ReferenceDataSetUrl.VB_Description = "Provides a link to some freely available sample data sets for comparing accuracy.  http://www.itl.nist.gov/div898/strd/."
  ' provided for easy reference when crosschecking validity of data
  ' this link has high-precision statistical data and results
  ' useful for testing results of specialized statistical calculations
  ReferenceDataSetUrl = "http://www.itl.nist.gov/div898/strd/"
End Property
